module Octopando
  class JiraClient < JIRA::Client
    DOMAIN = 'instructure.atlassian.net'

    def inspect
      "<Octopando::Jira::Client site: #{options[:site]}>"
    end

    def my_issues
      @my_issues ||= (
        my_issues_jql = "assignee = currentUser() AND resolution = Unresolved ORDER BY updatedDate DESC"
        issues = JIRA::Resource::Issue.jql(self, my_issues_jql)
        issues
      )
    end

    def self.from_keychain
      @keychain_client ||= (
        keychain_item = Keychain.default.internet_passwords.where(server: DOMAIN).first
        raise Octopando::KeyChainError, "#{DOMAIN} is in your keychain" unless keychain_item
        self.from_keychain_item(keychain_item) if keychain_item
      )
    end

    def self.from_keychain_item(keychain_item)
      options = {
                  :username => keychain_item.account,
                  :password => keychain_item.password,
                  :site     => "https://#{DOMAIN}",
                  :context_path => '',
                  :auth_type => :basic
                }
      self.new(options)
    end
  end
end
